/* Pango
 * pangopdflib.h: Routines for handling FreeType2 fonts with PDFlib library
 *
 * Copyright (C) 1999 Red Hat Software
 * Copyright (C) 2000 Tor Lillqvist
 * Copyright (C) 2001 Sun Microsystems
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __PANGOPDFLIB_H__
#define __PANGOPDFLIB_H__

#include <pdflib.h>

#include <ft2build.h>
#include FT_FREETYPE_H

#include <fontconfig/fontconfig.h>

#include <pango/pango-layout.h>
#include <pango/pango-attributes.h>
#include <pango/pango-xsl-attributes.h>

G_BEGIN_DECLS

typedef enum {
  PANGO_PDFLIB_FONT_EMBED_INVALID,
  PANGO_PDFLIB_FONT_EMBED_ALL,
  PANGO_PDFLIB_FONT_EMBED_NONBASE,
  PANGO_PDFLIB_FONT_EMBED_NONE
} PangoPDFLIBFontEmbed;

#define PANGO_RENDER_TYPE_PDFLIB "PangoRenderPDFLIB"

#define PANGO_TYPE_PDFLIB_FONT_MAP              (pango_pdflib_font_map_get_type ())
#define PANGO_PDFLIB_FONT_MAP(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_PDFLIB_FONT_MAP, PangoPDFLIBFontMap))
#define PANGO_PDFLIB_IS_FONT_MAP(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_PDFLIB_FONT_MAP))

typedef struct _PangoPDFLIBFontMap      PangoPDFLIBFontMap;
typedef struct _PangoPDFLIBLayout      PangoPDFLIBLayout;
typedef struct _PangoPDFLIBLayoutClass PangoPDFLIBLayoutClass;

typedef void (*PangoPDFLIBSubstituteFunc) (FcPattern *pattern,
                                        gpointer   data);

/* Calls for applications */
PangoContext      *pango_pdflib_get_context    (double dpi_x,
						double dpi_y);
PangoFontMap      *pango_pdflib_font_map_for_display (void);
void               pango_pdflib_shutdown_display     (void);


void           pango_pdflib_render             (PDF              *pdf_doc,
						const gchar  *layout_text,
						PangoLayoutRun   *run,
						PangoFont           *font,
						PangoGlyphString    *glyphs,
						gint               x,
						gint               y);
void           pango_pdflib_render_layout_line (PDF           *pdf_doc,
						const gchar   *layout_text,
						PangoLayoutLine  *line,
						gint              x,
						gint         y);
void 	       pango_pdflib_render_layout_lines (PDF         *pdf_doc,
						 PangoLayout *layout,
						 gint	      line_first,
						 gint         line_last,
						 gint         x, 
						 gint         y);
void           pango_pdflib_render_layout   (PDF              *pdf_doc,
					     PangoLayout      *layout,
					     gint              x, 
					     gint              y);

GType pango_pdflib_font_map_get_type (void);

PangoFontMap *pango_pdflib_font_map_new                    (void);
void          pango_pdflib_font_map_set_resolution         (PangoPDFLIBFontMap        *fontmap,
							 double                  dpi_x,
							 double                  dpi_y);
PangoPDFLIBFontEmbed pango_pdflib_font_map_get_font_embed (PangoPDFLIBFontMap *font_map);
void          pango_pdflib_font_map_set_font_embed (PangoPDFLIBFontMap      *font_map,
						    PangoPDFLIBFontEmbed font_embed);
void          pango_pdflib_font_map_set_default_substitute (PangoPDFLIBFontMap        *fontmap,
							    PangoPDFLIBSubstituteFunc  func,
							    gpointer                   data,
							    GDestroyNotify             notify);
void          pango_pdflib_font_map_substitute_changed     (PangoPDFLIBFontMap         *fontmap);
PangoContext *pango_pdflib_font_map_create_context         (PangoPDFLIBFontMap         *fontmap);

typedef enum
{
  PANGO_PDFLIB_DEBUG_NONE	= 0,
  PANGO_PDFLIB_DEBUG_DEBUG	= 1,
  PANGO_PDFLIB_DEBUG_MASK	= 0x01
} PangoPDFLIBDebugFlags;

PangoContext *pango_pdflib_font_map_create_context_with_debug_flags (PangoPDFLIBFontMap *fontmap,
								     PangoPDFLIBDebugFlags debug_flags);

void pango_pdflib_layout_set_line_height (PangoLayout *layout, gint line_height);
void pango_pdflib_layout_set_line_stacking_strategy (PangoLayout *layout,
						     PangoLineStackingStrategy line_stacking_strategy);

gint pango_pdflib_layout_get_item_attrs (PangoItem *item,
					 PangoAttrType first_attribute_type,
					 ...);

/* API for rendering modules
 */
PangoGlyph     pango_pdflib_get_unknown_glyph (PangoFont       *font);
int            pango_pdflib_font_get_kerning  (PangoFont       *font,
					    PangoGlyph       left,
					    PangoGlyph       right);
FT_Face        pango_pdflib_font_get_face     (PangoFont       *font);
PangoCoverage *pango_pdflib_font_get_coverage (PangoFont       *font,
					    PangoLanguage   *language);

G_END_DECLS

#endif /* __PANGOPDFLIB_H__ */
